;;; @ tex-misc.el - Miscellaneous utilility functions for TeX mode.
;;; 
;;; $Id: tex-misc.el,v 5.34 1993/08/05 10:00:43 amanda Exp $

(provide 'tex-misc)
(require 'tex-init)
(require 'easymenu)

;;; @@ Copyright
;;;
;;; Copyright 1991 Kresten Krab Thorup
;;; Copyright 1992,1993 Per Abrahamsen
;;; 
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 1, or (at your option)
;;; any later version.
;;; 
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;; 
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;;; @@ Syntax Table

(defvar TeX-mode-syntax-table (make-syntax-table)
  "Syntax table used while in TeX mode.")

 (make-variable-buffer-local 'TeX-mode-syntax-table)

(progn ; Define TeX-mode-syntax-table.
  (modify-syntax-entry (string-to-char TeX-esc)
		           "\\" TeX-mode-syntax-table)
  (modify-syntax-entry ?\f ">"  TeX-mode-syntax-table)
  (modify-syntax-entry ?\n ">"  TeX-mode-syntax-table)
  (modify-syntax-entry (string-to-char TeX-grop)
		           (concat "(" TeX-grcl)
		                TeX-mode-syntax-table)  
  (modify-syntax-entry (string-to-char TeX-grcl)
		           (concat ")" TeX-grop)
			        TeX-mode-syntax-table)  
  (modify-syntax-entry ?%  "<"  TeX-mode-syntax-table)
  (modify-syntax-entry ?\" "."  TeX-mode-syntax-table)
  (modify-syntax-entry ?&  "."  TeX-mode-syntax-table)
  (modify-syntax-entry ?_  "."  TeX-mode-syntax-table)
  (modify-syntax-entry ?@  "_"  TeX-mode-syntax-table)
  (modify-syntax-entry ?~  " "  TeX-mode-syntax-table)
  (modify-syntax-entry ?'  "w"  TeX-mode-syntax-table))

;;; @@ Keymap

(defvar TeX-mode-map nil
  "Keymap for common TeX and LaTeX commands.")

(if TeX-mode-map 
    ()
  (setq TeX-mode-map (make-sparse-keymap))

  ;; Standard
  (define-key TeX-mode-map "\177"     'backward-delete-char-untabify)
  (define-key TeX-mode-map "\C-c}"    'up-list)
  (define-key TeX-mode-map "\C-c#"    'TeX-normal-mode)
  (define-key TeX-mode-map "\C-c\C-n" 'TeX-normal-mode)
  (define-key TeX-mode-map "\C-c?"    'describe-mode)
  
  ;; From tex-misc.el
  (define-key TeX-mode-map "\""       'TeX-insert-quote)
  (define-key TeX-mode-map "$"        'TeX-insert-dollar)
  (define-key TeX-mode-map "\C-c{"    'TeX-insert-braces)
  (define-key TeX-mode-map "\C-c\C-f" 'TeX-font)

  (define-key TeX-mode-map "\C-c;"    'TeX-comment-region)
  (define-key TeX-mode-map "\C-c%"    'TeX-comment-paragraph)

  (define-key TeX-mode-map "\C-c'"    'TeX-comment-out-paragraph) ;*** Old way
  (define-key TeX-mode-map "\C-c:"    'TeX-un-comment-region) ;*** Old way
  (define-key TeX-mode-map "\C-c\""   'TeX-un-comment) ;*** Old way

  ;; From tex-cpl.el
  (define-key TeX-mode-map "\C-c\C-m" 'TeX-insert-macro)

  ;; From tex-buf.el
  (define-key TeX-mode-map "\C-c\C-r" 'TeX-command-region)
  (define-key TeX-mode-map "\C-c\C-c" 'TeX-command-master)
  (define-key TeX-mode-map "\C-c\C-k" 'TeX-kill-job)
  (define-key TeX-mode-map "\C-c\C-l" 'TeX-recenter-output-buffer)
  (define-key TeX-mode-map "\C-c^" 'TeX-home-buffer)

  ;; From tex-dbg.el
  (define-key TeX-mode-map "\C-c`"    'TeX-next-error)
  (define-key TeX-mode-map "\C-c\C-w" 'TeX-toggle-debug-boxes)
  ;;(define-key TeX-mode-map "\C-c@" 'TeX-toggle-debug-boxes) ;*** Stupid alias

  ;; From tex.cpl.el
  (define-key TeX-mode-map "\e\t"   'TeX-complete-symbol) ;*** Emacs 19 way
  (define-key TeX-mode-map "\C-c\t"   'TeX-complete-symbol))

(defvar plain-TeX-mode-map (copy-keymap TeX-mode-map)
  "Keymap used in plain TeX mode.")

(defun TeX-command-menu (name file)
  "Create menu NAME for each entry TeX-command-list for FILE."
  (append (list name)
	  (mapcar (function (lambda (entry) 
			      (vector (car entry)
				      (list 'TeX-command
					    (car entry)
					    (list 'quote file))
				      t)))
		  TeX-command-list)))

(easy-menu-define plain-TeX-mode-menu
    plain-TeX-mode-map
    "Menu used in plain TeX mode."
  (list "AUC TeX"
	["Macro..." TeX-insert-macro t]
	["Complete" TeX-complete-symbol t]
	(TeX-command-menu "Command on Master File" 'TeX-master-file)
	(TeX-command-menu "Command on Region" 'TeX-region-file)
	["Next Error" TeX-next-error t]
	["Kill Job" TeX-kill-job t]
	["Toggle debug of boxes" TeX-toggle-debug-boxes t]
	["Switch to original file" TeX-home-buffer t]
	["Recenter Output Buffer" TeX-recenter-output-buffer t]
	;; ["Uncomment" TeX-un-comment t]
	["Uncomment Region" TeX-un-comment-region t]
	;; ["Comment Out Paragraph" TeX-comment-out-paragraph t]
	;; ["Comment Paragraph" TeX-comment-paragraph t]
	["Comment Region" TeX-comment-region t]
	["Switch to Master file" TeX-home-buffer t]
	["Reset Buffer" TeX-normal-mode t]
	["Reset AUC TeX" (TeX-normal-mode t) "  C-u C-c C-n"]))

;;; @@ Simple Comments

(defun TeX-comment-out-region (level start end)
  "Insert LEVEL comment characters before each line in the region."
  (interactive "*p\nr")
  (save-excursion
    (goto-char start)
    (beginning-of-line)
    (setq start (point))
    (goto-char end)
    (end-of-line)
    (setq end (point))
    (save-restriction 
      (narrow-to-region end start)
      (goto-char (point-min))
      (while (not (eobp))
	(let ((count level))
	  (while (> count 0)
	    (insert comment-start)
	    (setq count (- count 1))))
	(forward-line 1)))))

(defun TeX-un-comment-region (level start end)
  "Remove up to LEVEL comment characters from each line in the region."
  (interactive "*p\nr") 
  (save-excursion
    (goto-char start)
    (beginning-of-line)
    (setq start (point))
    (goto-char end)
    (end-of-line)
    (setq end (point))
    (save-restriction 
      (narrow-to-region start end)
      (goto-char (point-min))
      (while (not (eobp))
	(let ((count level))
	  (while (and (> count 0)
		      (not (eobp))
		      (looking-at (regexp-quote comment-start)))
	    (delete-char 1)
	    (setq count (- count 1))))
	(forward-line 1)))))

(defun TeX-un-comment (level)
  "Delete up to LEVEL %'s from the beginning of each line in a comment."
  (interactive "*p")
  (save-excursion
    ; Find first comment line
    (re-search-backward (concat "^[^" comment-start "]") nil 'limit)
    (let ((beg (point)))
      (forward-line 1)
      ; Find last comment line
      (re-search-forward (concat "^[^" comment-start "]") nil 'limit)
      ; Uncomment region
      (TeX-un-comment-region level beg (point)))))

(defun TeX-comment-out-paragraph (level)
  "Inserts LEVEL %'s at the beginning of every line in the current paragraph."
  (interactive "*p")
  (save-excursion
    (end-of-line)
    (start-of-paragraph-text)
    (beginning-of-line)
    (let ((beg (point)))
      (end-of-paragraph-text)
      ;; (forward-line -1)
      (beginning-of-line)
      (TeX-comment-out-region level beg (point)))))

;;; @@ Smart Comments

(defun TeX-comment-region (level begin end)
  "Remove or insert comments from region.

Without an argument, insert one level of comments in all lines in the
current region.

With a C-u argument, uncomment one level of comments from the
current region.

With a positive numeric argument, insert that many comment characters
in all lines in the current region.

With a negative numeric argument, remove that many comment character
from the current region."
  (interactive "*P\nr")
  (cond ((null level)
	 (TeX-comment-out-region 1 begin end))
	((listp level)
	 (TeX-un-comment-region 1 begin end))
	((eq level '-)
	 (TeX-un-comment-region 1 begin end))
	((> level 0)
	 (TeX-comment-out-region level begin end))
	((< level 0)
	 (TeX-un-comment-region (- level) begin end))))

(defun TeX-comment-paragraph (level)
  "Remove or insert comments from paragraph.

Without an argument, insert one level of comments in all lines in the
current paragraph.

With a C-u argument, uncomment one level of comments from the
surrounding comment lines.

With a positive numeric argument, insert that many comment characters
in all lines in the current paragraph.

With a negative numeric argument, remove that many comment character
from the surrounding comment lines."
  (interactive "*P")
  (cond ((null level)
	 (TeX-comment-out-paragraph 1))
	((listp level)
	 (TeX-un-comment 1))
	((eq level '-)
	 (TeX-un-comment 1))
	((> level 0)
	 (TeX-comment-out-paragraph level))
	((< level 0)
	 (TeX-un-comment (- level)))))

;;; @@ Indentation

(defun TeX-brace-count-line ()
  "Count number of open/closed braces."
  (if (looking-at "%")
      0
    (save-excursion
      (save-restriction
	(let ((count 0))
	  (narrow-to-region (point)
			    (save-excursion
			      (re-search-forward "[^\\\\]%\\|\n\\|\\'")
			      (backward-char)
			      (point)))

	  (if (looking-at "\\({\\|}\\)")
	    (if (string= "{" (TeX-match-buffer 1))
		(setq count (+ TeX-brace-indent-level count))
	      (setq count (- count TeX-brace-indent-level))))
	      
	  (while (re-search-forward "[^\\\\]\\({\\|}\\)" nil t)
	    (if (string= "{" (TeX-match-buffer 1))
		(setq count (+ TeX-brace-indent-level count))
	      (setq count (- count TeX-brace-indent-level)))
	    (if (looking-at "\\({\\|}\\)")
		(if (string= "{" (TeX-match-buffer 1))
		    (setq count (+ TeX-brace-indent-level count))
		  (setq count (- count TeX-brace-indent-level)))))
	  count)))))

(defvar TeX-brace-indent-level 2
  "*The level of indentation produced by a open brace.")

(defun TeX-comment-indent ()
  (if (looking-at "%%%")
      (current-column)
    (skip-chars-backward " \t")
    (max (if (bolp) 0 (1+ (current-column)))
	 comment-column)))

;;; @@ Fonts

(defvar TeX-font-list '((?\C-b "{\\bf " "}")
			(?\C-c "{\\sc " "}")
			(?\C-e "{\\em " "\\/}")
			(?\C-i "{\\it " "\\/}")
			(?\C-r "{\\rm " "}")
			(?\C-s "{\\sl " "\\/}")
			(?\C-t "{\\tt " "}")
			(?\C-d "" "" t))
  "*List of fonts used by TeX-font.

Each entry is a list with three elements.  The first element is the
key to active the font.  The secont element is the string to insert
before point, and the third element is the string to insert after
point.  An optional fourth element means always replace if not nil.")

(defun TeX-font (replace what)
  "Insert template for font change command.

If REPLACE is not nil, replace current font.  WHAT determines the font
to use, as specified by TeX-font-list."
  (interactive "*P\nc")
  (let* ((entry (assoc what TeX-font-list)))
    (setq replace (or replace (nth 3 entry)))
    (cond ((null entry)
	   (error "No font for defined `%c'." what))
	  ((not replace)
	   (insert (nth 1 entry))
	   (save-excursion
	     (insert (nth 2 entry))))
	  (t
	   (TeX-font-replace (nth 1 entry) (nth 2 entry))))))

(defun TeX-font-replace (start end)
  "Replace font specification around point with START and END."
  (save-excursion
    (while (not (looking-at "{\\\\[a-zA-Z]+ "))
      (up-list -1))
    (forward-sexp)
    (save-excursion
      (replace-match start t t))
    (if (save-excursion
	  (backward-char 3)
	  (if (looking-at (regexp-quote "\\/}"))
	      (progn
		(delete-char 3)
		nil)
	    t))
	(delete-backward-char 1))
    (insert end)))

;;; @@ Dollars
;;;
;;; Originally stolen from VorTeX.
;;; Copyright (C) 1986, 1987, 1988 Pehong Chen (phc@renoir.berkeley.edu)

(defvar TeX-dollar-sign ?$
  "*Character user to enter and leaver math mode in TeX.")

(defconst TeX-dollar-string (char-to-string TeX-dollar-sign))

(defconst TeX-dollar-regexp 
  (concat "^" (regexp-quote TeX-dollar-string) "\\|[^" TeX-esc "]"
	  (regexp-quote TeX-dollar-string)))
  
(defvar TeX-dollar-list nil)
  (make-variable-buffer-local 'TeX-match-dollar-on)

(defvar TeX-par-start nil)
  (make-variable-buffer-local 'TeX-par-start)

(defvar TeX-par-end nil)
  (make-variable-buffer-local 'TeX-par-end)

(defvar TeX-symbol-marker nil)

(defvar TeX-symbol-marker-pos 0)

(defun TeX-bouncing-point (m)
  (save-excursion
    (if (pos-visible-in-window-p)
	(sit-for 1)
      (let* ((pos1 (point))
             (pos2 (+ pos1 m))
             (sym (buffer-substring pos1 pos2))
             (msg1 (progn (beginning-of-line) (buffer-substring (point) pos1)))
             (msg2 (progn (end-of-line) (buffer-substring pos2 (point)))))
        (message "%s`%s'%s" msg1 sym msg2)))))

(defun TeX-locate-delimiter (pos sym symlst)
  (let ((marker nil)
        (marker-pos 0)
        (pair t)
        (head nil))
    (catch 'loop
      (while symlst
        (setq marker (car symlst))
        (setq marker-pos (1- (marker-position marker)))
        (if (and (/= pos marker-pos) (= (char-after marker-pos) sym))
	    (if (> pos marker-pos)
		(progn
		  (setq TeX-symbol-marker-pos marker-pos)
		  (setq TeX-symbol-marker marker) 
		  (setq head (cons marker head))
		  (setq pair (not pair)))
	      (if pair (setq TeX-symbol-marker nil))
	      (throw 'loop (append (reverse head)
				   (cons (set-marker (make-marker) (1+ pos)) 
					 symlst)))))
        (setq symlst (cdr symlst)))
      (if pair (setq TeX-symbol-marker nil))
      (reverse (cons (set-marker (make-marker) (1+ pos)) head)))))

(defun TeX-dollar-verify ()
  ;; Verify if the current paragraph is the same as last.
  ;; If so, do nothing, otherwise reset TeX-par-start and TeX-par-end and
  ;; reconstruct the symbol-list.
  (let ((start (save-excursion
                 (if (re-search-backward paragraph-separate nil t)
		     (point)
                   1)))
        (end (save-excursion
               (if (re-search-forward paragraph-separate nil t)
		   (1+ (point))
                 (1+ (point-max)))))
        (init nil))
    (if (null TeX-par-start)
	(setq TeX-par-start (set-marker (make-marker) 1)))
    (if (/= (marker-position TeX-par-start) start)
	(progn
	  (set-marker TeX-par-start start)
	  (setq init t)))
    (if (null TeX-par-end)
	(setq TeX-par-end (set-marker (make-marker) 1)))
    (if (/= (marker-position TeX-par-end) end)
	(progn
	  (set-marker TeX-par-end end)
	  (setq init t)))
    (if init
	(save-excursion
	  (setq TeX-dollar-list nil)
	  (goto-char start)
	  (while (re-search-forward TeX-dollar-regexp end t)
	    (setq TeX-dollar-list
		  (append TeX-dollar-list
			  (list (set-marker (make-marker)
					    (if (= (following-char)
						   TeX-dollar-sign)
						(progn
						  (forward-char 1)
						  (point))
					      (point)))))))))))

(defun TeX-insert-dollar (&optional arg)
  "Insert dollar sign.  

Show matching dollar sign if this dollar sign end the TeX math mode.  
Ensure double dollar signs match up correctly by inserting extra
dollar signs when needed.

With optional ARG, insert that many dollar signs."
  (interactive "P")
  (if arg
      (let ((count (prefix-numeric-value arg)))
	(if (listp arg)
	    (self-insert-command 1)	;C-u always inserts just one
	  (self-insert-command count)))
    (let ((pc (preceding-char))
	  (pos (point))
	  (pt (point))
	  (single t))
      (TeX-dollar-verify)
      (if (= pc (string-to-char TeX-esc))
	  (insert TeX-dollar-sign)
	(if (= pc TeX-dollar-sign)
	    (progn
	      (setq single nil)
	      (if (and (> pos 2) (= (char-after (- pos 2)) TeX-dollar-sign))
		  (setq pt (1- pos))	; Doesn't echo 3rd $, if $$ already
		(backward-char 1) 
		(insert TeX-dollar-sign)
		(goto-char (1+ pos))))
	  (insert TeX-dollar-sign))
	(setq TeX-dollar-list
	      (TeX-locate-delimiter pt TeX-dollar-sign TeX-dollar-list))
	(if TeX-symbol-marker
	    (save-excursion
	      (goto-char TeX-symbol-marker-pos)
	      (if (and (= (preceding-char) TeX-dollar-sign)
		       (/= (char-after (- (point) 2)) TeX-dollar-sign))
		  (progn
		    (backward-char 1)
		    (if single
			(save-excursion
			  (goto-char pos)
			  (insert TeX-dollar-sign))))	; $$foo$`$'
		(if (not single)
		    (progn
		      (insert TeX-dollar-sign) ; `$'$foo$$
		      (backward-char 1))))
	      (TeX-bouncing-point (if single 1 2))))))))

;;; @@ Miscellaneous

(defun TeX-normal-mode (arg)
  "Remove all information about this buffer, and apply the style hooks again.
Save buffer first including style information.
With optional argument, also reload the style hooks."
  (interactive "*P")
  (if arg
      (setq TeX-style-hook-list nil))
  (require 'tex-auto)
  (let ((TeX-auto-save t))
    (save-buffer))
  (normal-mode)
  (TeX-update-style))

(defvar TeX-open-quote "``"
  "*String inserted by typing \\[TeX-insert-quote] to open a quotation.")

(defvar TeX-close-quote "''"
  "*String inserted by typing \\[TeX-insert-quote] to close a quotation.")

(defun TeX-insert-quote (arg)
  "Insert the appropriate quote marks for TeX.
Inserts the value of `tex-open-quote' (normally ``) or `tex-close-quote'
\(normally '') depending on the context.  With prefix argument, always
inserts \" characters."
  (interactive "*P")
  (if arg
      (let ((count (prefix-numeric-value arg)))
	(if (listp arg)
	    (self-insert-command 1)	;C-u always inserts just one
	  (self-insert-command count)))
    (TeX-update-style)
    (insert (cond ((bobp) TeX-open-quote)
		  ((= (preceding-char) (string-to-char TeX-esc))
		   ?\")
		  ((= (preceding-char) ?\")
		   ?\")
		  ((save-excursion
		     (forward-char (- (length TeX-open-quote)))
		     (looking-at (regexp-quote TeX-open-quote)))
		   (delete-backward-char (length TeX-open-quote))
		   ?\")
		  ((save-excursion
		     (forward-char (- (length TeX-close-quote)))
		     (looking-at (regexp-quote TeX-close-quote)))
		   (delete-backward-char (length TeX-close-quote))
		   ?\")
		  ((save-excursion
		     (forward-char -1)
		     (looking-at "[ \t\n]\\|\\s("))
		   TeX-open-quote)
		  (t
		   TeX-close-quote)))))

;;; For the sake of BibTeX...
(fset 'tex-insert-quote 'TeX-insert-quote)

(defun TeX-insert-braces ()
  "Make a pair of braces and be poised to type inside of them."
  (interactive "*")
  (insert TeX-grop)
  (save-excursion
    (insert TeX-grcl)))

;;; @@ Emacs

(run-hooks 'TeX-after-tex-misc-hook)

;;; Local Variables:
;;; mode: emacs-lisp
;;; mode: outline-minor
;;; outline-regexp: ";;; @+\\|(......"
;;; End:

