;;; @ tex-site.el - Site specific variables.
;;;
;;; $Id: tex-site.el,v 5.50 1993/07/27 01:12:39 amanda Exp $

(provide 'tex-site)

;;; @@ Description
;;;
;;; This file contains variables to be customized for a given site.
;;;
;;; It also contains all necessary autoloads, so the user can simple
;;; enable AUC TeX by putting (load "tex-site.el") in his .emacs
;;; file. 
;;;
;;; If you are afraid of the delay of loading this file, you can
;;; copy the sections Path and Autoloads to your file verbatim instead.

;;; @@ Copyright
;;;
;;; Copyright (C) 1991 Kresten Krab Thorup 
;;; Copyright (C) 1993 Per Abrahamsen 
;;; 
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 1, or (at your option)
;;; any later version.
;;; 
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;; 
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;;; @@ Path
;;;
;;; Change these for your site.

(defvar TeX-lisp-directory "/usr/local/lib/emacs/site-lisp/auctex/"
  "*The directory where the AUC TeX lisp files are located.

The directory name *must* end with a slash.")

(defvar TeX-macro-global
  '("/usr/local/lib/tex/inputs/" "/usr/local/lib/tex/generate/")
  "*Directories containing the sites TeX macro files and style files.

The directory names *must* end with a slash.")

;;; @@ Autoloads
;;;
;;; This set up everything to be loaded as needed.
;;; No need for changes in this section.

(setq load-path (cons TeX-lisp-directory load-path))

;;; Used by BibTeX mode
(autoload 'TeX-insert-quote "tex-misc" nil t)
(autoload 'tex-insert-quote "tex-misc" nil t)

(autoload 'BibTeX-auto-store "tex-auto" nil t)
(defvar bibtex-mode-hook nil)
(or (memq 'BibTeX-auto-store bibtex-mode-hook)
    (setq bibtex-mode-hook
	  (cons 'BibTeX-auto-store bibtex-mode-hook)))

;;; TeX and LaTeX major modes
(autoload 'tex-mode "tex-init" "Major mode for editing (La)TeX text." t)
(autoload 'latex-mode "tex-init" "Major mode for editing LaTeX text." t)
(autoload 'slitex-mode "tex-init" "Major mode for editing SliTeX text." t)
(autoload 'foiltex-mode "tex-init" "Major mode for editing FoilTeX text." t)
(autoload 'ams-tex-mode "tex-init" "Major mode for editing AmSTeX text." t)
(autoload 'ams-latex-mode "tex-init" "Major mode for editing AmSLaTeX text." t)
(autoload 'plain-tex-mode "tex-init" "Major mode for editing TeX text." t)
(autoload 'japanese-latex-mode "tex-jp" "Japanese LaTeX mode." t)
(autoload 'japanese-slitex-mode "tex-jp" "Japanese SliTeX mode." t)
(autoload 'japanese-plain-tex-mode "tex-jp" "Japanese TeX mode." t)

;;; Generate AUTO files
(autoload 'TeX-auto-generate-global "tex-auto" "Generate site auto files." t)
(autoload 'TeX-auto-generate "tex-auto" "Generate specific auto files." t)

;;; Outline Minor Mode
(autoload 'outline-minor-mode "min-out" "Toggle outline minor mode." t)
(make-variable-buffer-local 'outline-prefix-char)
(setq-default outline-prefix-char "\C-c\C-o") ; Change to get a better prefix
(make-variable-buffer-local 'outline-regexp)
(setq-default outline-regexp "[*\^l]+")
(make-variable-buffer-local 'outline-level-function)
(setq-default outline-level-function 'outline-level-default)

;;; Mapping the keyboard layout
(autoload 'keyboard-mode "min-key" "Toggle keyboard-mode." t)
(autoload 'keyboard-query-mapping "min-key" "Query keyboard-mode." t)
(autoload 'convert-character-query "min-key" "Convert between encodings." t)
(autoload 'convert-character-encoding "min-key"
          "Convert between encodings." nil)

;;; Accent's in LaTeX
(setq LaTeX-dead-keys '(?' ?` ?\" ?^ ?~)) ; Remove those you don't need
(autoload 'LaTeX-dead-mode "ltx-dead" "Toggle LaTeX-dead-mode." t)

;;; Mathematical macros in TeX
(setq LaTeX-math-abbrev-prefix "`") ; Change if you like ` as is.
(autoload 'LaTeX-math-mode "tex-math" "Toggle math mode." t)

;;; Other minor modes
(autoload 'auto-indent-mode "min-ind" "Toggle auto-indent-mode." t)
(autoload 'auto-ispell-mode "min-ispl" "Toggle auto-ispell-mode." t)

;;; @@ From tex-buf.el

;;; The following four variables are the names of the commands `known'
;;; to AUC TeX, you must to change these if you change their names
;;; in TeX-command-list.  The names of the TeX formatters must match
;;; the values used in the `format' directory.

(defvar TeX-command-BibTeX "BibTeX"
  "*The name of the BibTeX entry in TeX-command-list.")

  (make-variable-buffer-local 'TeX-command-BibTeX)

(defvar TeX-command-Show "View"
  "*The default command to show (view or print) a TeX file.
Must be the car of an entry in TeX-command-list.")

  (make-variable-buffer-local 'TeX-command-Show)

;;; This is the major configuration variable.  Most sites will only
;;; need to change the second string in each entry, which is the name
;;; of a command to send to the shell.  If you use other formatters
;;; like AMSLaTeX or AMSTeX, you can add those to the list.  See
;;; TeX-expand-list for a description of the % escapes

(defvar TeX-print-command "dvips %s -P%p"
  "*Command used to print a file. 

First %p is expanded to the printer name, then ordinary expansion is
performed as specified in TeX-expand-list.")

(defvar TeX-queue-command "lpq -P%p"
  "*Command used to print a file. 

First %p is expanded to the printer name, then ordinary expansion is
performed as specified in TeX-expand-list.")

(defvar TeX-command-list
  (list (list "TeX" "tex '\\nonstopmode\\input %t'" 'TeX-TeX-hook nil t)
	(list "LaTeX" "latex '\\nonstopmode\\input{%t}'" 'TeX-LaTeX-hook nil t)
	(list "SliTeX" "slitex '\\nonstopmode\\input{%t}'"
	      'TeX-LaTeX-hook nil t)  
	(if window-system
	    (list "View" "%v " 'TeX-background-hook t nil)
	  (list "View" "dvi2tty -q -w 132 %s " 'TeX-command-hook t nil))
	(list "Print" "%p " 'TeX-command-hook t nil)
	(list "Queue" "%q" 'TeX-background-hook nil nil)
	(list "File" "dvips %d -o %f " 'TeX-command-hook t nil)
	(list "BibTeX" "bibtex %s" 'TeX-BibTeX-hook nil nil)
	(list "Index" "makeindex %s" 'TeX-command-hook nil nil)
	(list "Check" "lacheck %s" 'TeX-compile-hook nil t)
	(list "Spell" "<ignored>" 'TeX-ispell-hook nil t)
	(list "Other" "" 'TeX-command-hook t nil)
	;; Not part of standard TeX.
;;;	(list "AmSTeX" "amstex '\\nonstopmode\\input{%t}'"
;;;	      'TeX-TeX-hook nil t)
;;;	(list "AmSLaTeX" "amslatex '\\nonstopmode\\input{%t}'"
;;;	      'TeX-LaTeX-hook nil t)
;;;	(list "FoilTeX" "foiltex '\\nonstopmode\\input{%t}'"
;;;	      'TeX-LaTeX-hook nil t)
	;; The next entries support Japanese TeX.
;;;	(list "jTeX" "jtex '\\nonstopmode\\input %t'" 'TeX-TeX-hook nil t)
;;;	(list "jLaTeX" "jlatex '\\nonstopmode\\input{%t}'"
;;;	      'TeX-LaTeX-hook nil t)
;;;	(list "jSliTeX" "jslitex '\\nonstopmode\\input{%t}'"
;;;	      'TeX-LaTeX-hook nil t)
;;;	(list "jBibTeX" "jbibtex %s" 'TeX-BibTeX-hook nil nil)
	;; Add your own entries here :-)
	)

  "*List of commands to execute on the current document.

Each element is a list, whose first element is the name of the command
as it will be presented to the user.  

The second element is the string handed to the shell after being
expanded. The expansion is done using the information found in
TeX-expand-list. 

The third element is the function which actually start the process.
Several such hooks has been defined:

TeX-command-hook: Start up the process and show the output in a
separate buffer.  Check that there is not two commands running for the
same file.  Return the process object. 

TeX-format-hook: As TeX-command-hook, but assume the output is created
by a TeX macro package.  Return the process object. 

TeX-TeX-hook: For TeX output.

TeX-LaTeX-hook: For LaTeX output.

TeX-BibTeX-hook: For BibTeX output.

TeX-compile-hook: Use `compile' to run the process.  

TeX-shell-hook: Use `shell-command' to run the process.

Tex-discard-hook: Start the process in the background, discarding its
output.

TeX-background-hook: Start the process in the background, show output
in other window.

TeX-dviout-hook: Special hook for the Japanese dviout previewer for
PC-9801.

To create your own hook, define a function taking three arguments: The
name of the command, the command string, and the name of the file to
process.  It might be useful to use TeX-command-hook in order to
create an asynchronous process.

If the fourth element is non-nil, the user will get a chance to
modify the expanded string.

If the fifth element is non-nil, the TeX-region file will be rebuild
before the command is started.")


;;; Enter the names of the printers available at your site, or nil if
;;; you only have one printer.

(defvar TeX-printer-list
  '(("Local" "dvips -f %s | lpr" "lpq")
    ("lw") ("ps"))
  "*List of available printers.

The first element of each entry is the printer name.

The second element is the command used to print to this
printer.  It defaults to the value of TeX-print-command.

The third element is the command used to examine the print queue for
this printer.  It defaults to the value of TeX-queue-command.

Any occurence of `%p' in the second or third element is expanded to
the printer name given in the first element, then ordinary expansion
is performed as specified in TeX-expand-list.")

;;; The name of the most used printer.  

(defvar TeX-printer-default (or (getenv "PRINTER")
				(and TeX-printer-list
				     (car (car TeX-printer-list)))
				"lw")
  "*Default printer to use with TeX-command.")

;;; You may want special options to the view command depending on the
;;; style optios.

(defvar TeX-view-style '(("^epsf$" "ghostview %f")
			 ("^a5$" "xdvi %d -paper a5")
			 ("^landscape$" "xdvi %d -paper a4r -s 4")
			 ("." "xdvi %d"))
  "*List of style options and view options.

If the first element (a regular expresion) matches the name of one of
the style files, any occurrence of the string %v in a command in
TeX-command-list will be replaced with the second element.  The first
match is used, if no match is found the %v is replaced with the empty
string.")

;;; Ispell supports multiple languages, and so does TeX.  
;;;
;;; This requires ispell 3.0 installes correctly, if you can't get it
;;; to work, don't tell me about it.  I can't make it work either.

(defvar TeX-ispell-style '(("^dutch$" "dutch")
			   ("^dk$" "danish")
			   ("." "english"))
  "List of style options and ispell dictionaries.

If the first element (a regular expresion) matches the name of one of the
style files, the buffer-local variable ispell:dictionary will be replaced
with the second element.  The first match is used.")

(make-variable-buffer-local 'ispell:dictionary)

(defun ispell-tex-buffer-p ()
  (and (boundp 'TeX-auto-update) TeX-auto-update))

(setq ispell-enable-tex-parser t)

(defun TeX-ispell-hook (command string file)
  "Hook for running ispell on a buffer."
  (let ((dict (TeX-style-check TeX-ispell-style)))
    (if (fboundp 'ispell-change-dictionary)
	(ispell-change-dictionary dict))
    (ispell-buffer)))

;;; This is the list of expansion for the commands in
;;; TeX-command-list.  Not likely to be changed, but you may e.g. want
;;; to handle .ps files. 

(defvar TeX-expand-list 
  (list (list "%p" 'TeX-printer-query)	;%p must be the first entry
	(list "%q" (function (lambda ()
			       (TeX-printer-query TeX-queue-command 2))))
	(list "%v" 'TeX-style-check TeX-view-style)
	(list "%s" 'file)
	(list "%t" 'file "tex")
	(list "%d" 'file "dvi")
	(list "%f" 'file "ps"))
  "*List of expansion strings for TeX command names.

Each entry is a list with two or more elements.  The first element is
the string to be expanded.  The second element is the name of a
function returning the expanded string when called with the remaining
elements as arguments.  The special value `file' will be expanded to
the name of the file being processed, with an optional extension.")

;;; Change the following for non-UNIX boxes:
;;;
;;; For OS/2 TeX-shell should be "cmd".
;;; For DEMACS, use shell-file-name and shell-command-option.

(defvar TeX-shell "/bin/sh"
  "Name of shell used to parse TeX commands.")

(defvar TeX-shell-command-option "-c"
  "Shell argument indicating that next argument is the command.")

;;; @@ Coding systems and Language
;;;
;;; This is for Japanese TeX.

(if (boundp 'MULE)
    (progn
      (defvar TeX-japanese-process-input-coding-system *euc-japan*
	"TeX-process' coding system with standard input.")
      (defvar TeX-japanese-process-output-coding-system *junet*
	"TeX-process' coding system with standard output.")))

(if (boundp 'NEMACS)
    (defvar TeX-process-kanji-code 2
      "TeX-process' kanji code with standard I/O.
0:No-conversion  1:Shift-JIS  2:JIS  3:EUC/AT&T/DEC"))

(defvar TeX-debug-language '(("JTEX" "dbg-jp")
			     ("JLATEX" "dbg-jp")
			     ("JSLITEX" "dbg-jp")
			     ("." "dbg-eng"))
  "*List of style options and languages for debug messages.  If the
first element (a regular expresion) matches the name of one of the
style files, the error messages will be loaded from the file specified
by the second element.  The first match will be used.")


;;; @@ Emacs

(run-hooks 'TeX-after-tex-site-hook)

;;; Local Variables:
;;; mode: emacs-lisp
;;; mode: outline-minor
;;; outline-regexp: ";;; @+\\|(......"
;;; End:
